BLOCK = 6

function love.load()
	c = love.graphics.newCanvas()
	reset()
end

function reset()
	maze = {[0] = {[0] = true}}
	frontier = {{-1, 0}, {0, -1}, {1, 0}, {0, 1}}
	c:clear()
	t = 0
end

function getxy(pos, vertical, direction)
	local x = pos[1]
	local y = pos[2]
	if vertical then
		y = y + direction
	else
		x = x + direction
	end
	return x, y
end

function getend(pos, vertical, direction)
	local x, y = getxy(pos, vertical, direction)
	return maze[x] and maze[x][y]
end

function setend(pos, vertical, direction)
	local x, y = getxy(pos, vertical, direction)
	if not maze[x] then maze[x] = {} end
	maze[x][y] = true
end

function addifnew(x, y, ox, oy)
	if x ~= ox or y ~= oy then
		table.insert(frontier, {x, y})
	end
end

function addsuccessors(pos, vertical, direction)
	local x, y = getxy(pos, vertical, direction)
	local ox, oy = unpack(pos)
	addifnew(x - 1, y, ox, oy)
	addifnew(x, y - 1, ox, oy)
	addifnew(x + 1, y, ox, oy)
	addifnew(x, y + 1, ox, oy)
end

function sstep()
	local new = table.remove(frontier, math.random(#frontier))
	if not new then return end
	local vertical = new[1]%2 == 0
	if not getend(new, vertical, -1) then
		setend(new, vertical, -1)
		addsuccessors(new, vertical, -1)
	elseif not getend(new, vertical, 1) then
		setend(new, vertical, 1)
		addsuccessors(new, vertical, 1)
	else
		return false
	end
	
	love.graphics.setCanvas(c)
	local x1, x2, y1, y2 
	if vertical then
		x1 = new[1]
		w = 1
		y1 = new[2] - 1
		h = 3
	else
		x1 = new[1] - 1
		w = 3
		y1 = new[2]
		h = 1
	end
	love.graphics.rectangle('fill', x1 * BLOCK + 400, y1 * BLOCK + 300, w * BLOCK, h * BLOCK)
	love.graphics.setCanvas()
	return true
end

function step()
	for i=1, 1 do
		while not sstep() do
		end
	end
end

function love.update(dt)
	t = t + dt
	while t >= .005 do
		t = t - .005
		step()
	end
end

function love.draw()
	love.graphics.draw(c, 0, 0)
end

function love.keypressed(key, u)
	if key == ' ' then
		reset()
	elseif key == 'escape' then
		love.event.quit()
	end	
end
